## $Id: export.py 12084 2014-11-28 09:54:10Z henrik $
##
## Copyright (C) 2015 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""Exporters for student related stuff.
"""
import grok
from datetime import datetime
from waeup.uniben.students.interfaces import (
    ICustomStudent,
    ICustomStudentStudyCourse,
    ICustomStudentStudyLevel,
    ICustomCourseTicket,
    ICustomStudentOnlinePayment,
    IMedicalHistory)
from kofacustom.nigeria.students.export import (
    NigeriaStudentExporter,
    NigeriaStudentStudyCourseExporter,
    NigeriaStudentStudyLevelExporter,
    NigeriaCourseTicketExporter,
    NigeriaStudentPaymentExporter,
    NigeriaDataForBursaryExporter,
    )
from waeup.kofa.students.export import (
    SchoolFeePaymentsOverviewExporter, StudentExporterBase)
from waeup.kofa.utils.helpers import iface_names

class CustomStudentExporter(NigeriaStudentExporter):
    """Exporter for Students.
    """

    fields = tuple(sorted(iface_names(
        ICustomStudent, omit=['loggerInfo']))) + (
        'password', 'state', 'history', 'certcode', 'is_postgrad',
        'current_level', 'current_session')

class CustomStudentStudyCourseExporter(NigeriaStudentStudyCourseExporter):
    """Exporter for StudentStudyCourses.
    """

    fields = tuple(
        sorted(iface_names(ICustomStudentStudyCourse))) + ('student_id',)

class CustomStudentStudyLevelExporter(NigeriaStudentStudyLevelExporter):
    """Exporter for StudentStudyLevels.
    """
    #: Fieldnames considered by this exporter
    fields = tuple(sorted(iface_names(
        ICustomStudentStudyLevel))) + (
        'student_id', 'number_of_tickets','certcode')

class CustomCourseTicketExporter(NigeriaCourseTicketExporter):
    """Exporter for CourseTickets.
    """

    fields = tuple(sorted(iface_names(ICustomCourseTicket) +
        ['level', 'code', 'level_session'])) + ('student_id',
        'certcode', 'display_fullname')

class CustomStudentPaymentExporter(NigeriaStudentPaymentExporter):
    """Exporter for OnlinePayment instances.
    """

    fields = tuple(
        sorted(iface_names(
            ICustomStudentOnlinePayment, exclude_attribs=False,
            omit=['display_item']))) + (
            'student_id','state','current_session')

class CustomDataForBursaryExporter(NigeriaDataForBursaryExporter):
    """
    """

    fields = tuple(
        sorted(iface_names(
            ICustomStudentOnlinePayment, exclude_attribs=False,
            omit=['display_item', 'certificate', 'student']))) + (
            'student_id','matric_number','reg_number',
            'firstname', 'middlename', 'lastname',
            'state','current_session',
            'entry_session', 'entry_mode',
            'faccode', 'depcode','certcode')

class CustomSchoolFeePaymentsOverviewExporter(SchoolFeePaymentsOverviewExporter):

    curr_year = datetime.now().year
    year_range = range(curr_year - 14, curr_year + 1) # 3 more years in Uniben
    year_range_tuple = tuple([str(year) for year in year_range])

    fields = ('student_id', 'matric_number', 'firstname', 'middlename',
        'lastname', 'state', 'certcode', 'faccode', 'depcode', 'is_postgrad',
        'current_level', 'current_session', 'current_mode',
        'entry_session', 'reg_number'
        ) + year_range_tuple

class MedicalHistoryExporter(grok.GlobalUtility, StudentExporterBase):
    """
    """
    grok.name('medicalhistory')

    fields = tuple(
        sorted(iface_names(
            IMedicalHistory, exclude_attribs=False,))) + (
            'student_id','display_fullname', 'matric_number', 'faccode',
            'depcode', 'state','current_session', 'current_level')
    title = 'Medical Questionnaire Data'

    def mangle_value(self, value, name, context=None):
        """The mangler determines the titles of faculty, department
        and certificate. It also computes the path of passport image file
        stored in the filesystem.
        """
        if context is not None:
            student = context.student
            if name in ('student_id','display_fullname',
                'matric_number', 'faccode',
                'depcode', 'state','current_session',
                'current_level') and student is not None:
                value = getattr(student, name, None)
        return super(
            MedicalHistoryExporter, self).mangle_value(
            value, name, context=context)
